function [X, lambda, phi] = bicluster_update_X(m,n,Y,U,beta,tol,lambda,phi)

% subroutine of clustersdp_ADM

%% parameters
M = 20;
em = ones(m,1); % All-ones vectors.
en = ones(n,1);
sigma = 1e-4;
stepinit = 1;


%% initialization
% matrix to fit X to.
Q = Y + U/beta;
nQ = norm(Q,'fro')^2/2;
% Lambda.
tmp = [(repmat(lambda,1,m) + repmat(lambda',m,1))/2, zeros(m,n); zeros(n,m), (repmat(phi,1,n) + repmat(phi',n,1))/2] ;
% Initial X.
X = max(Q - tmp,0);
% Initial optimal values.
pval = norm(X - Q,'fro')^2/2 - nQ;
dval = norm(X,'fro')^2/2 + sum(lambda) + sum(phi);

% Gradients.
grad_l = em - X(1:m, 1:m)*em;
grad_p = en - X(m+1: m+n, m+1: m+n)*en;

d1 = -inf*ones(M,1);
d1(1) = dval;
dmax = max(d1);
iter = 1;

% % gradient check
% v = randn(n,1);
% lambdanew = lambda + 1e-7*v;
% tmp = (repmat(lambdanew,1,n) + repmat(lambdanew',n,1))/2;
% Xnew = max(Q - tmp,0);
% dnew = norm(Xnew,'fro')^2/2 + sum(lambdanew);
% [(dnew - dval)/1e-7 grad'*v]
% pause

while 1 == 1
    
    if abs(pval + dval)/max(abs(pval),1) < tol && max(max(-grad_l),max(-grad_p)) <= tol
        break
    end
    
    % Search direction.
    dir_l = max(lambda - stepinit*grad_l,0) - lambda; % Direction for lambda.
    dir_p = max(phi - stepinit*grad_p,0) - phi; % Direction for lambda.
    dderiv = grad_l'*dir_l + grad_p'*dir_p;
    
    % Update variables.
    stepsize = 1;
    lambdanew = lambda + stepsize*dir_l;
    phinew = phi + stepsize*dir_p;
    tmp = [(repmat(lambdanew,1,m) + repmat(lambdanew',m,1))/2, zeros(m,n); zeros(n,m), (repmat(phinew,1,n) + repmat(phinew',n,1))/2] ;
    X = max(Q - tmp,0);
    dval = norm(X,'fro')^2/2 + sum(lambdanew) + sum(phinew);
    
    while dval > dmax + sigma*stepsize*dderiv && stepsize > 1e-10
        stepsize = stepsize/2;
        lambdanew = lambda + stepsize*dir_l;
        phinew = phi + stepsize*dir_p;
        tmp = [(repmat(lambdanew,1,m) + repmat(lambdanew',m,1))/2, zeros(m,n); zeros(n,m), (repmat(phinew,1,n) + repmat(phinew',n,1))/2 ];
        X = max(Q - tmp,0);
        dval = norm(X,'fro')^2/2 + sum(lambdanew) + sum(phinew);
    end
    
    if stepsize <= 1e-10
        break
    end
    
    gradnew_l = em - X(1:m,1:m)*em;
    gradnew_p = en - X(m+1: m+n, m+1: m+n)*en;
    
    dsl = lambdanew - lambda;
    dsp = phinew - phi;
    dgl = gradnew_l - grad_l;
    dgp = gradnew_p - grad_p;
    
    ds = [dsl;dsp]; dg = [dgl; dgp];
    
    stepinit = min(max(norm(ds)^2/(ds'*dg),1e-8),1e3);
    
    % update variables
    lambda = lambdanew;
    phi = phinew;
    grad_l = gradnew_l;
    grad_p = gradnew_p;
    d1(mod(iter,M)+1) = dval;
    dmax = max(d1);
    pval = norm(X - Q,'fro')^2/2 - nQ;
    
    %   fprintf(' pval, dval, stepsize %g, %g, %g\n', pval, dval, stepsize)
    
    iter = iter + 1;
    
end

X = (X+X')/2;

% fprintf(' spg terminates: iter %5d\n', iter)



