function [X,lambda] = cluster_update_X(n,Y,U,beta,tol,lambda)

% subroutine of clustersdp_ADM

% parameters
M = 20;
e = ones(n,1);
sigma = 1e-4;
stepinit = 1;

% initialization
Q = Y + U/beta;
nQ = norm(Q,'fro')^2/2;
tmp = (repmat(lambda,1,n) + repmat(lambda',n,1))/2;
X = max(Q - tmp,0);
pval = norm(X - Q,'fro')^2/2 - nQ;
dval = norm(X,'fro')^2/2 + sum(lambda);

grad = e - X*e;
d1 = -inf*ones(M,1);
d1(1) = dval;
dmax = max(d1);
iter = 1;

% % gradient check
% v = randn(n,1);
% lambdanew = lambda + 1e-7*v;
% tmp = (repmat(lambdanew,1,n) + repmat(lambdanew',n,1))/2;
% Xnew = max(Q - tmp,0);
% dnew = norm(Xnew,'fro')^2/2 + sum(lambdanew);
% [(dnew - dval)/1e-7 grad'*v]
% pause

while 1 == 1
  
  if abs(pval + dval)/max(abs(pval),1) < tol && max(-grad) <= tol
    break
  end
  
  dir = max(lambda - stepinit*grad,0) - lambda;
  dderiv = grad'*dir;
  
  stepsize = 1;
  lambdanew = lambda + stepsize*dir;
  tmp = (repmat(lambdanew,1,n) + repmat(lambdanew',n,1))/2;
  X = max(Q - tmp,0);
  dval = norm(X,'fro')^2/2 + sum(lambdanew);
  
  while dval > dmax + sigma*stepsize*dderiv && stepsize > 1e-10
    stepsize = stepsize/2;
    lambdanew = lambda + stepsize*dir;
    tmp = (repmat(lambdanew,1,n) + repmat(lambdanew',n,1))/2;
    X = max(Q - tmp,0);
    dval = norm(X,'fro')^2/2 + sum(lambdanew);
  end
  
  if stepsize <= 1e-10
    break
  end
  
  gradnew = e - X*e;
  
  ds = lambdanew - lambda;
  dg = gradnew - grad;
  
  stepinit = min(max(norm(ds)^2/(ds'*dg),1e-8),1e3);
  
  % update variables
  lambda = lambdanew;
  grad = gradnew;
  d1(mod(iter,M)+1) = dval;
  dmax = max(d1);
  pval = norm(X - Q,'fro')^2/2 - nQ;
  
%   fprintf(' pval, dval, stepsize %g, %g, %g\n', pval, dval, stepsize)
  
  iter = iter + 1;
  
end

% fprintf(' spg terminates: iter %5d\n', iter)
  
  
  
  