function [r,k] = gen_clustersizes( N,rhat )

% Given total number of nodes N and desired minimum cluster size rhat, 
% generates list of cluster sizes r = [r1,..., rk] containing N nodes of
% size roughly rhat (with no outliers).

%% Initialize clusters.

% # of clusters is roughly N/rhat.
k = floor(N/rhat);

% Start with rhat nodes in each cluster.
r = rhat*ones(k,1);

% Have N - krhat leftover.
left = N - k*rhat;

%% Evenly distribute remainder of nodes to clusters.

% Initialize position of cluster to be updated.
pos = 1;

% While leftover nodes exist.
while left >=1
    
    % Increment current cluster.
    r(pos) = r(pos) + 1;
    
    % Decrease number of leftover.
    left = left - 1;
    
    % Update position to be incremented.
    if pos == (k- 1)
        pos = 1;    % Only update first k-1, so min(ri) = rhat.
    else
        pos = pos + 1; % Else move to next in list.
    end
    
end

