function [W, W0] = planted_wkdb(m,n, a, b)

% Generates a N-node graph with k clusters of size at least rmin.
% Start with all-ones blocks representing cliques indexed by each clusters and then
% delete clique edges, and add nonclique edges independently with probability p.

%% Initialize G:

M = sum(m);
N = sum(n);

% Generate random matrix.
R = 0.25*randn(M, N);
% Set off-diagonal blocks.
W = b + R;

W0 = zeros(M,N);


%% Determine block structure of V wrt r.

% # of cluster.
k =length(m);

u_beg = ones(k,1); % Initialize positions of start point of each planted clique.
v_beg = ones(k,1);
% Set positions.
for i=2:k+1
    % Position is last position plus size of last clique.
   u_beg(i) = u_beg(i-1) + m(i-1); 
   v_beg(i) = v_beg(i-1) + n(i-1); 
end


%% Insert edges for planted dense subgraphs.


% Make dense blocks
for i=1:k
    % Get G.
       
    mind = u_beg(i):(u_beg(i+1)-1);
    nind = v_beg(i):(v_beg(i+1)-1);
    
    W(mind, nind) = a + R(mind, nind);    
    W0(mind, nind) = ones(m(i), n(i))/sqrt(m(i)*n(i));
end

