function [W,W0, k] = planted_wkdc(r,a, b)

% Generates a kernel matrix W for a  planted k-disjoint-clique subgraph
% indexed by the vector r (r1:rk) are clique sizes),

% Get total number of nodes.
N = sum(r);
% Number of planted cliques.
k = length(r);

%% Initialize W:
% Generate random matrix.
R = 0.25*randn(N);
% Set off-diagonal blocks.
W = b + R;

W0 = zeros(N);

%% Add planted cliques.

% Determine block structure of V wrt r.

c_beg = ones(k+1,1); % Initialize positions of start point of each planted clique.

% Set positions.
for i=2:k+1
    % Position is last position plus size of last clique.
    c_beg(i) = c_beg(i-1) + r(i-1); 
end

% Make diagonal-blocks blocks.
for i=1:k    
    W(c_beg(i):(c_beg(i+1)-1), c_beg(i):(c_beg(i+1)-1)) = a + R(c_beg(i):(c_beg(i+1)-1), c_beg(i):(c_beg(i+1)-1));
    W0(c_beg(i):(c_beg(i+1)-1), c_beg(i):(c_beg(i+1)-1)) = ones(r(i))/r(i);
end

%% Symmetrize W.
W = (W+W')/2;
